<?php

/**
 * @file
 * Administration functionality for the PoolParty Taxonomy Manager.
 */

/**
 * List all saved PoolParty Taxonomy Manager configurations.
 *
 * @return array
 *   A renderable array of the list of configurations.
 */
function pp_taxonomy_manager_admin_list() {
  $output = array();

  $output['pp_taxonomy_manager_title'] = array(
    '#type' => 'markup',
    '#markup' => '<h3 class="semantic-connector-table-title">' . t('PoolParty Taxonomy Manager configurations') . '</h3>',
  );

  $rows = array();
  $configs = pp_taxonomy_manager_config_load_multiple();
  /* @var $config PPTaxonomyManagerConfig */
  foreach ($configs as $config) {
    // Set the title.
    $title = '<div class="semantic-connector-led" data-server-id="' . $config->getConnection()
        ->getId() . '" data-server-type="pp-server" title="' . t('Checking service') . '"></div>';
    $title .= $config->getTitle();

    // Set the link to the PoolParty server.
    $pp_server = l($config->getConnection()
      ->getTitle(), $config->getConnection()
        ->getUrl() . '/PoolParty', array('attributes' => array('target' => '_blank')));

    // Set the project label.
    $settings = $config->getConfig();
    if ($settings['root_level'] != 'project') {
      $project_id = $config->getProjectID();
      $connection_config = $config->getConnection()->getConfig();
      $project_label = t('Project label not found');
      if (isset($connection_config['projects'])) {
        foreach ($connection_config['projects'] as $project) {
          if ($project->id == $project_id) {
            $project_label = $project->title;
            break;
          }
        }
      }
    }
    else {
      $project_label = '-';
    }

    // Create the list of the interconnected taxonomies.
    $configuration = $config->getConfig();
    $vocabularies = taxonomy_vocabulary_load_multiple(array_keys($configuration['taxonomies']));
    $taxonomies = array();
    foreach ($vocabularies as $vocabulary) {
      $taxonomies[] = l($vocabulary->name, 'admin/structure/taxonomy/' . $vocabulary->machine_name);
    }
    $taxonomies = empty($taxonomies) ? '<div class="semantic-connector-italic">' . t('not yet set') . '</div>' : theme('item_list', array('items' => $taxonomies));

    // Set the operations.
    $actions = array(
      l(t('Edit'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId()),
      l(t('Delete'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/delete'),
    );

    // Create the row.
    $rows[] = array(
      $title,
      $pp_server,
      $project_label,
      $taxonomies,
      implode(' | ', $actions),
    );
  }

  $output['pp_taxonomy_manager'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('Name'),
      t('PoolParty server'),
      t('Selected project'),
      t('Interconnected taxonomies'),
      t('Operations'),
    ),
    '#rows' => $rows,
    '#empty' => t('There are currently no PoolParty Taxonomy Manager configurations available.'),
    '#attributes' => array(
      'id' => 'pp-taxonomy-manager-configurations-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $output['#attached']['js'] = array(libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js');
  }

  return $output;
}


/**
 * Ajax callback function for checking if a new PoolParty server is available.
 */
function pp_taxonomy_manager_new_available($form, $form_state) {
  $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
  $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';

  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Create a new connection (without saving) with the current form data.
    $connection = SemanticConnector::getConnection('pp_server');
    $connection->setUrl($form_state['values']['url']);
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));

    $availability = $connection->getApi('PPT')->available();
    if (isset($availability['message']) && !empty($availability['message'])) {
      return '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . $availability['message'] . '</div>';
    }
    else {
      return $availability['success'] ? $available : $not_available;
    }
  }

  return $not_available;
}

/**
 * The form for a PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   The PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_connection_form($form, &$form_state, PPTaxonomyManagerConfig $config = NULL) {
  if (!is_null($config) && !empty($config)) {
    $is_configuration_new = FALSE;
    $form['tmid'] = array(
      '#type' => 'hidden',
      '#value' => $config->getId(),
    );

    $form['title'] = array(
      '#type' => 'hidden',
      '#value' => $config->getTitle(),
    );
  }
  else {
    $is_configuration_new = TRUE;
    $config = new PPTaxonomyManagerConfig();

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#description' => t('Name of the PoolParty Taxonomy Manager configuration.'),
      '#size' => 35,
      '#maxlength' => 255,
      '#default_value' => $config->getTitle(),
      '#required' => TRUE,
      '#validated' => TRUE,
    );
  }

  // Container: Server settings.
  $form['server_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('1. Select the PoolParty server to use'),
  );

  $connections = SemanticConnector::getConnectionsByType('pp_server');
  if (!empty($connections)) {
    $connection_options = array();
    /* @var $connection SemanticConnectorConnection */
    foreach ($connections as $connection) {
      $credentials = $connection->getCredentials();
      $connection_options[implode('|', array(
        $connection->getTitle(),
        $connection->getUrl(),
        $credentials['username'],
        $credentials['password'],
      ))] = $connection->getTitle();
    }
    $form['server_settings']['load_connection'] = array(
      '#type' => 'select',
      '#title' => t('Load an available PoolParty server'),
      '#options' => $connection_options,
      '#empty_option' => '',
      '#default_value' => '',
    );
  }

  // Container: Connection details.
  $form['server_settings']['connection_details'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection details'),
  );

  $connection = $config->getConnection();
  $form['server_settings']['connection_details']['connection_id'] = array(
    '#type' => 'hidden',
    '#value' => $connection->getId(),
  );

  $form['server_settings']['connection_details']['server_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Server title'),
    '#description' => t('A short title for the server below.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $connection->getTitle(),
    '#required' => TRUE,
  );

  $form['server_settings']['connection_details']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('URL, where the PoolParty server runs.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $connection->getUrl(),
    '#required' => TRUE,
  );

  $credentials = $connection->getCredentials();
  $form['server_settings']['connection_details']['credentials'] = array(
    '#type' => 'fieldset',
    '#title' => t('Credentials'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['server_settings']['connection_details']['credentials']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Name of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 60,
    '#default_value' => $credentials['username'],
  );
  $form['server_settings']['connection_details']['credentials']['pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#description' => t('Password of a user for the credentials.'),
    '#size' => 35,
    '#maxlength' => 128,
    '#default_value' => $credentials['password'],
  );

  $form['server_settings']['health_check'] = array(
    '#type' => 'button',
    '#value' => t('Health check'),
    '#ajax' => array(
      'callback' => 'pp_taxonomy_manager_new_available',
      'wrapper' => 'health_info',
      'method' => 'replace',
      'effect' => 'slide',
    ),
  );

  if ($is_configuration_new) {
    $markup = '<div id="health_info">' . t('Click to check if the server is available.') . '</div>';
  }
  else {
    $available = '<div id="health_info" class="available"><div class="semantic-connector-led led-green" title="Service available"></div>' . t('The server is available.') . '</div>';
    $not_available = '<div id="health_info" class="not-available"><div class="semantic-connector-led led-red" title="Service NOT available"></div>' . t('The server is not available or the credentials are incorrect.') . '</div>';
    $markup = $connection->available() ? $available : $not_available;
  }
  $form['server_settings']['health_info'] = array(
    '#markup' => $markup,
  );

  // Container: Level settings.
  $form['level_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('2. Select the taxonomy root level'),
  );

  $settings = $config->getConfig();
  $form['level_settings']['root_level'] = array(
    '#type' => 'radios',
    '#options' => array(
      'project' => t('Every taxonomy becomes a !rootlevel.', array('!rootlevel' => '<b>' . t('PoolParty Project') . '</b>')),
      'conceptscheme' => t('Every taxonomy becomes a !rootlevel in a single project.', array('!rootlevel' => '<b>' . t('Concept Scheme') . '</b>')),
    ),
    '#default_value' => $settings['root_level'],
    '#disabled' => !$is_configuration_new,
  );

  // Container: Project loading.
  $form['project_load'] = array(
    '#type' => 'fieldset',
    '#title' => t('3. Load the projects'),
    '#states' => array(
      'visible' => array(
        ':input[name="root_level"]' => array('value' => 'conceptscheme'),
      ),
    ),
  );

  $form['project_load']['load_projects'] = array(
    '#type' => 'button',
    '#value' => 'Load projects',
    '#ajax' => array(
      'event' => 'click',
      'callback' => 'pp_taxonomy_manager_connection_form_get_projects',
      'wrapper' => 'projects-replace',
    ),
  );

  // Container: Project selection.
  $form['project_select'] = array(
    '#type' => 'fieldset',
    '#title' => t('4. Select the project to use'),
    '#description' => t('Note: In case this list is still empty after clicking the "Load projects" button make sure that a connection to the PoolParty server can be established and check the rights of your selected user inside PoolParty.'),
    '#states' => array(
      'visible' => array(
        ':input[name="root_level"]' => array('value' => 'conceptscheme'),
      ),
    ),
  );

  // Get the project options for the currently configured PoolParty server.
  $project_options = array();
  if (!$is_configuration_new) {
    $projects = $connection->getApi('PPT')->getProjects();
    $skip_projects = _pp_taxonomy_manager_get_used_projects($connection->getUrl(), array($config->getId()));
    foreach ($projects as $project) {
      // If the project is not used yet, add it to the list.
      if (!isset($skip_projects[$project->id])) {
        $project_options[$project->id] = $project->title;
      }
    }
  }
  // Configuration set admin page.
  $form['project_select']['project'] = array(
    '#type' => 'select',
    '#title' => 'Select a project',
    '#prefix' => '<div id="projects-replace">',
    '#suffix' => '</div>',
    '#options' => $project_options,
    '#default_value' => (!$is_configuration_new ? $config->getProjectId() : NULL),
    '#states' => array(
      'required' => array(
        ':input[name="root_level"]' => array('value' => 'conceptscheme'),
      ),
    ),
    '#validated' => TRUE,
  );

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => ($is_configuration_new ? t('Save and edit details') : t('Save')),
    '#prefix' => '<div class="admin-form-submit-buttons" style="margin-top: 40px;">',
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => ($is_configuration_new ? 'admin/config/semantic-drupal/pp-taxonomy-manager' : 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId()),
    '#suffix' => '</div>',
  );

  // Add CSS and JS files
  $form['#attached'] = array(
    'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
  );

  return $form;
}


/**
 * Ajax callback function to get a project select list.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 *
 * @return array
 *   The select form element containing the project options for the current
 *   PoolParty server connection.
 */
function pp_taxonomy_manager_connection_form_get_projects($form, $form_state) {
  $projects_element = $form['project_select']['project'];

  $project_options = array();
  if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
    // Create a new connection (without saving) with the current form data.
    $connection = SemanticConnector::getConnection('pp_server');
    $connection->setUrl($form_state['values']['url']);
    $connection->setCredentials(array(
      'username' => $form_state['values']['name'],
      'password' => $form_state['values']['pass'],
    ));

    $skip_projects = _pp_taxonomy_manager_get_used_projects($form_state['values']['url'], (isset($form_state['values']['tmid']) ? array($form_state['values']['tmid']) : array()));
    $projects = $connection->getApi('PPT')->getProjects();
    foreach ($projects as $project) {
      // If the project is not used yet, add it to the list.
      if (!isset($skip_projects[$project->id])) {
        $project_options[$project->id] = $project->title;
      }
    }
  }

  $projects_element['#options'] = $project_options;
  return $projects_element;
}

/**
 * The validation handler for the PoolParty Taxonomy Manager connection form.
 */
function pp_taxonomy_manager_connection_form_validate($form, &$form_state) {
  // Only do project validation during the save-operation, not during
  // AJAX-requests like the health check of the server.
  if ($form_state['triggering_element']['#parents'][0] == 'save') {
    // A title is required.
    if (!isset($form_state['values']['tmid']) && empty($form_state['values']['title'])) {
      form_set_error('title', t('Name field is required.'));
    }

    if (isset($form_state['values']['url']) && valid_url($form_state['values']['url'], TRUE)) {
      // Create a new connection (without saving) with the current form data.
      $connection = SemanticConnector::getConnection('pp_server');
      $connection->setUrl($form_state['values']['url']);
      $connection->setCredentials(array(
        'username' => $form_state['values']['name'],
        'password' => $form_state['values']['pass'],
      ));
      // Check if the connection can be reached.
      if (!$connection->available()) {
        form_set_error('url', t('The selected connection can not be reached.'));
      }

      $root_level = $form_state['values']['root_level'];
      if ($root_level != 'project') {
        // A project needs to be selected.
        if (empty($form_state['values']['project'])) {
          form_set_error('project', t('Please select a project.'));
        }
        // And it has to be a valid project (one that is available on the connected
        // PoolParty server).
        else {
          $projects = $connection->getApi('PPT')->getProjects();
          $project_is_valid = FALSE;
          foreach ($projects as $project) {
            if ($project->id == $form_state['values']['project']) {
              $project_is_valid = TRUE;
              break;
            }
          }
          if (!$project_is_valid) {
            form_set_error('project', t('The selected project is not available on the given PoolParty server configuration.'));
          }
        }
      }
    }
    else {
      form_set_error('url', t('The field URL must be a valid URL.'));
    }
  }
}

/**
 * The submit handler for the PowerTagging connection form.
 */
function pp_taxonomy_manager_connection_form_submit($form, &$form_state) {
  // Always create a new connection, if URL and type are the same the old one
  // will be used anyway.
  $connection = SemanticConnector::createConnection('pp_server', $form_state['values']['url'], $form_state['values']['server_title'], array(
    'username' => $form_state['values']['name'],
    'password' => $form_state['values']['pass'],
  ));

  // Update an existing configuration.
  if (isset($form_state['values']['tmid'])) {
    $config = new PPTaxonomyManagerConfig($form_state['values']['tmid']);
    $config->setTitle($form_state['values']['title']);
    $config->setConnectionId($connection->getId());
    if ($form_state['values']['root_level'] != 'project') {
      $config->setProjectId($form_state['values']['project']);
    }
    $tmid = $config->save();
    drupal_set_message(t('The connection for PoolParty Taxonomy Manager configuration %title has been updated.', array('%title' => $form_state['values']['title'])));
  }
  // Create a new configuration set.
  else {
    $config = new PPTaxonomyManagerConfig();
    $config->setTitle($form_state['values']['title']);
    $config->setConnectionId($connection->getId());

    // Set the project ID.
    $root_level = $form_state['values']['root_level'];
    if ($root_level == 'project') {
      $config->setProjectId(NULL);
    }
    else {
      $config->setProjectId($form_state['values']['project']);
    }

    // Create the initial config.
    $settings = array(
      'root_level' => $root_level,
    );
    $config->setConfig($settings);

    // Save the PP Taxonomy Manager configuration.
    $tmid = $config->save();
    drupal_set_message(t('The PoolParty Taxonomy Manager configuration %title has been created.', array('%title' => $form_state['values']['title'])));
  }

  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid;
}

/**
 * Get a list of project IDs that are already used by PoolParty Taxonomy
 * Manager configurations.
 *
 * @param string $connection_url
 *   optional: a URL to filter the configurations with.
 * @param array $ignored_config_ids
 *   optional: An array of IDs of PoolParty Taxonomy Manager configurations
 *   to ignore in the checks.
 *
 * @return array
 *   An associative array of used project IDs (project ID => config title).
 */
function _pp_taxonomy_manager_get_used_projects($connection_url = NULL, $ignored_config_ids = array()) {
  $project_ids = array();
  $existing_configs = pp_taxonomy_manager_config_load_multiple();
  /** @var \PPTaxonomyManagerConfig $existing_config */
  foreach ($existing_configs as $existing_config) {
    if (!in_array($existing_config->getId(), $ignored_config_ids) && (is_null($connection_url) || $connection_url == $existing_config->getConnection()->getUrl())) {
      $existing_config_settings = $existing_config->getConfig();
      if ($existing_config_settings['root_level'] == 'conceptscheme') {
        $project_ids[$existing_config->getProjectId()] = $existing_config->getTitle();
      }
      else {
        foreach (array_values($existing_config_settings['taxonomies']) as $project_id) {
          $project_ids[$project_id] = $existing_config->getTitle();
        }
      }
    }
  }

  return $project_ids;
}

/**
 * A form to edit an existing PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_configuration_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  $form['tmid'] = array(
    '#type' => 'hidden',
    '#value' => $config->getId(),
  );

  // Get the configuration of the selected PoolParty Taxonomy Manager.
  $settings = $config->getConfig();

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Name of the PowerTagging configuration.'),
    '#size' => 35,
    '#maxlength' => 255,
    '#default_value' => $config->getTitle(),
    '#required' => TRUE,
  );

  // Get the project title of the currently configured project.
  $project_title = '';
  if ($settings['root_level'] != 'project') {
    $project_title = '<invalid project selected>';
    $pp_server_projects = $config->getConnection()
      ->getApi('PPT')
      ->getProjects();
    foreach ($pp_server_projects as $pp_server_project) {
      if ($pp_server_project->id == $config->getProjectId()) {
        $project_title = $pp_server_project->title;
        break;
      }
    }
  }

  // Add information about the connection.
  $connection_markup = '';
  // Check the PoolParty server version if required.
  if (variable_get('semantic_connector_version_checking', TRUE)) {
    $api_version_info = $config->getConnection()->getVersionInfo('PPT');
    if (version_compare($api_version_info['installed_version'], $api_version_info['latest_version'], '<')) {
      $connection_markup .= '<div class="messages warning">' . t('The connected PoolParty server is not up to date. You are currently running version %installedversion, upgrade to version %latestversion to enjoy the new features.', array(
          '%installedversion' => $api_version_info['installed_version'],
          '%latestversion' => $api_version_info['latest_version'],
        )) . '</div>';
    }
  }
  $connection_markup .= '<p id="pp-taxonomy-manager-connection-info">' . t('Connected PoolParty server') . ': <b>' . $config->getConnection()
      ->getTitle() . ' (' . $config->getConnection()->getUrl() . ')</b><br />'
    . t('Selected taxonomy root level') . ': <b>' . ($settings['root_level'] == 'project' ? t('Project') : t('Concept Scheme')) . '</b><br />'
    . (!empty($project_title) ? t('Selected project') . ': <b>' . $project_title . '</b><br />' : '')
    . l(t('Change the connected PoolParty server or project'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/edit_connection') . '</p>';
  $form['pp_connection_markup'] = array(
    '#type' => 'markup',
    '#markup' => $connection_markup,
  );

  // Get all available Drupal taxonomies.
  $taxonomies = taxonomy_get_vocabularies();

  // Get all taxonomies that are connected with a PowerTagging configuration.
  $powertaggings = array();
  $taxonomy_powertagging = array();
  if (module_exists('powertagging')) {
    $powertagging_configs = powertagging_config_load();
    foreach ($powertagging_configs as $powertagging_config) {
      $powertaggings[$powertagging_config->powertagging_id] = $powertagging_config;
      if (isset($powertagging_config->config['projects'][$powertagging_config->project_id]['taxonomy_id'])) {
        $vid = $powertagging_config->config['projects'][$powertagging_config->project_id]['taxonomy_id'];
        $taxonomy_powertagging[$vid][] = $powertagging_config->powertagging_id;
      }
    }
  }

  // Create a table with all existing and possible connections between Drupal
  // taxonomies and PoolParty concepts schemes for the selected project.

  // Create the table rows for all available concept schemes / projects not yet connected.
  $connected = array();
  $powertagging_ids = array();
  $rows = array();
  $connected_rows = array();
  // Load the projects.
  if ($settings['root_level'] == 'project') {
    $projects = $config->getConnection()
      ->getApi('PPT')
      ->getProjects();

    $used_projects = _pp_taxonomy_manager_get_used_projects($config->getConnection()->getUrl(), array($config->getId()));
    foreach ($projects as $project) {
      // The project is not yet used by a different configuration.
      if (!isset($used_projects[$project->id])) {
        $operations = array();
        if (in_array($project->id, $settings['taxonomies'])) {
          $connected[$project->id] = $project;
          continue;
        }

        // Check if the project is in use by a PowerTagging configuration.
        $powertagging_id = 0;
        foreach ($powertaggings as $powertagging_config) {
          if ($config->getConnection()->getUrl() == $powertagging_config->connection->getUrl() && $project->id == $powertagging_config->project_id) {
            $powertagging_id = $powertagging_config->powertagging_id;
            break;
          }
        }

        if ($powertagging_id == 0) {
          $operations[] = '&lArr; ' . l(t('Import into Drupal'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $project->id);
          $rows[] = array(
            '<div class="semantic-connector-italic">' . t('not yet connected') . '</div>',
            implode(' | ', $operations),
            l($project->title, $project->id, array('attributes' => array('title' => (isset($project->description) && !empty($project->description)) ? $project->description : NULL))),
          );
        }
        else {
          $powertagging_ids[$project->id] = $powertagging_id;
          $powertagging_config = $powertaggings[$powertagging_id];
          $taxonomy = $taxonomies[$powertagging_config->config['projects'][$powertagging_config->project_id]['taxonomy_id']];
          $operations[] = '&lArr; ' . l(t('Make complete taxonomy'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/powertagging-taxonomy-update/' . $powertagging_id);
          $rows[] = array(
            l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))) . t(' (created by PowerTagging configuration "%powertaggingtitle")', array('%powertaggingtitle' => $powertagging_config->title)),
            implode(' | ', $operations),
            l($project->title, $project->id, array('attributes' => array('title' => (isset($project->description) && !empty($project->description)) ? $project->description : NULL))),
          );
        }
      }
      // The project is already used by a different configuration.
      else {
        $rows[] = array(
          '-',
          '<div class="semantic-connector-italic">' . t('Already in use by PoolParty Taxonomy Manager configuration "%config_title"', array('%config_title' => $used_projects[$project->id])) . '</div>',
          l($project->title, $project->id, array('attributes' => array('title' => (isset($project->description) && !empty($project->description)) ? $project->description : NULL))),
        );
      }
    }
  }
  // Load the Concept Schemes.
  else {
    $concept_schemes = $config->getConnection()
      ->getApi('PPT')
      ->getConceptSchemes($config->getProjectId());

    foreach ($concept_schemes as $scheme) {
      $operations = array();
      if (in_array($scheme->uri, $settings['taxonomies'])) {
        $connected[$scheme->uri] = $scheme;
        continue;
      }
      $operations[] = '&lArr; ' . l(t('Import into Drupal'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $scheme->uri);
      $rows[] = array(
        '<div class="semantic-connector-italic">' . t('not yet connected') . '</div>',
        implode(' | ', $operations),
        l($scheme->title, $scheme->uri, array('attributes' => array('title' => (isset($scheme->descriptions) && !empty($scheme->descriptions)) ? $scheme->descriptions[0] : NULL))),
      );
    }
  }

  // Check which taxonomies are already used by other PoolParty Taxonomy
  // Manager configurations.
  $used_taxonomies = array();
  $taxonomy_manager_configs = pp_taxonomy_manager_config_load_multiple();
  /** @var \PPTaxonomyManagerConfig $taxonomy_manager_config */
  foreach ($taxonomy_manager_configs as $taxonomy_manager_config) {
    if ($taxonomy_manager_config->getId() != $config->getId()) {
      $taxonomy_manager_config_settings = $taxonomy_manager_config->getConfig();
      foreach (array_keys($taxonomy_manager_config_settings['taxonomies']) as $vid) {
        $used_taxonomies[$vid] = $taxonomy_manager_config->getTitle();
      }
    }
  }

  // Create the table rows for all connected and disconnected Drupal taxonomies.
  foreach ($taxonomies as $taxonomy) {
    $operations = array();
    // The taxonomy is not used by a different configuration.
    if (!isset($used_taxonomies[$taxonomy->vid])) {
      if (isset($settings['taxonomies'][$taxonomy->vid])) {
        $last_log = $config->getLastLog($taxonomy->vid);
        $sync_required = FALSE;

        // Create rows from connected taxonomies.
        if (isset($connected[$settings['taxonomies'][$taxonomy->vid]])) {
          $root_object = $connected[$settings['taxonomies'][$taxonomy->vid]];
          $powertagging_id = isset($powertagging_ids[$settings['taxonomies'][$taxonomy->vid]]) ? $powertagging_ids[$settings['taxonomies'][$taxonomy->vid]] : 0;

          // Check if a background process is running.
          $batch_urls = array(
            'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $root_object->uri,
            'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/export/' . $taxonomy->vid,
            'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/powertagging-taxonomy-update/' . $powertagging_id,
            'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/sync/' . $taxonomy->vid,
          );
          $bid = pp_taxonomy_manager_batch_running($batch_urls);
          if ($bid !== FALSE) {
            $operations[] = '&lArr; ' . l(t('Sync in progress'), 'batch', array('query' => array('op' => 'start', 'id' => $bid))) . ' &rArr;';
          }
          else {
            $operations[] = '&lArr; ' . l(t('Sync from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/sync/' . $taxonomy->vid);
            $operations[] = l(t('Disconnect from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/disconnect/' . $taxonomy->vid) . ' &rArr;';
          }
          $concept_scheme = l($root_object->title, $root_object->uri, array(
            'absolut' => TRUE,
            'attributes' => array('title' => isset($root_object->description) ? $root_object->description : (isset($root_object->descriptions) && !empty($root_object->descriptions) ? $root_object->descriptions[0] : '')),
          ));

          // Check if there were any PoolPart side changes.
          $changes = _pp_taxonomy_manager_check_pp_changes($config, ($settings['root_level'] == 'project' ? $root_object->id : $config->getProjectId()), $taxonomy->vid);
          $sync_required = !empty($changes);
        }
        else {
          $root_level = ($settings['root_level'] == 'project') ? 'project' : 'concept scheme';
          $operations[] = l(t('Disconnect from PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/disconnect/' . $taxonomy->vid) . ' &rArr;';
          $concept_scheme = t('The !root_level could not be found in PoolParty.<br />Make sure to delete the connection if the !root_level was deleted on purpose.', array('!root_level' => $root_level));
        }
        $taxonomy_link = l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description)));
        if (isset($taxonomy_powertagging[$taxonomy->vid])) {
          $powertagging_config_labels = array();
          foreach ($taxonomy_powertagging[$taxonomy->vid] as $powertagging_id) {
            $powertagging_config = $powertaggings[$powertagging_id];
            $powertagging_config_labels[] = $powertagging_config->title;
          }
          $taxonomy_link .= t(' (used by @configpart "%powertaggingtitle")', array('@configpart' => format_plural(count($powertagging_config_labels), 'PowerTagging configuration', 'PowerTagging configurations'), '%powertaggingtitle' => implode('", "', $powertagging_config_labels)));
        }

        $connected_rows[] = array(
          $taxonomy_link,
          implode(' | ', $operations),
          $concept_scheme,
          (($last_log !== FALSE) ? t('!sorttimestampstarted: @starttime!brfinished: @endtime!brby: !username', array('@starttime' => format_date($last_log['start_time'], 'short'), '@endtime' => format_date($last_log['end_time'], 'short'), '!username' => l($last_log['name'], 'user/' . $last_log['uid']), '!br' => '<br />', '!sorttimestamp' => '<span style="display:none;">' . $last_log['start_time'] . '</span>')) : ''),
          (($sync_required) ? '<b>' . t('YES') . '</b>' : t('no')),
        );
      }
      elseif (!isset($taxonomy_powertagging[$taxonomy->vid])) {
        // Create rows from disconnected taxonomies.
        $operations[] = l(t('Export to PoolParty'), 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/export/' . $taxonomy->vid) . ' &rArr;';
        $rows[] = array(
          l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
          implode(' | ', $operations),
          '<div class="semantic-connector-italic">' . t('not yet connected') . '</div>',
        );
      }
    }
    // The taxonomy is already used by a different configuration.
    else {
      $rows[] = array(
        l($taxonomy->name, 'admin/structure/taxonomy/' . $taxonomy->machine_name, array('attributes' => array('title' => $taxonomy->description))),
        '<div class="semantic-connector-italic">' . t('Already in use by PoolParty Taxonomy Manager configuration "%config_title"', array('%config_title' => $used_taxonomies[$taxonomy->vid])) . '</div>',
        '-',
      );
    }
  }

  if (empty($connected_rows) && empty($rows)) {
    drupal_set_message(t('There are currently no Drupal taxonomies or %rootlevel which could be synchronized.', array('%rootlevel' => ($settings['root_level'] == 'project' ? t('PoolParty projects') : t('PoolParty concept schemes')))), 'warning');
  }
  else {
    // Check if the tablesorter library is available.
    $tablesorter_path = 'sites/all/libraries/tablesorter';
    if (module_exists('libraries')) {
      $tablesorter_path = libraries_get_path('tablesorter');
    }
    $tablesorter_available = file_exists($tablesorter_path . '/jquery.tablesorter.min.js');

    // Already synchronized Drupal taxonomies.
    if (!empty($connected_rows)) {
      // Create the table for the connections.
      $table = array();
      $table['connections'] = array(
        '#theme' => 'table',
        '#header' => array(
          t('Drupal taxonomy'),
          t('Operations'),
          $settings['root_level'] == 'project' ? t('PoolParty project') : t('PoolParty concept scheme'),
          t('Last Sync'),
          t('Sync required'),
        ),
        '#rows' => $connected_rows,
        '#attributes' => array(
          'id' => 'pp-taxonomy-manager-synced-table',
          'class' => array('semantic-connector-tablesorter'),
        ),
        '#attached' => array(
          'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
        ),
      );

      // Add the tablesorter library if available.
      if ($tablesorter_available) {
        $table['#attached']['js'] = array($tablesorter_path . '/jquery.tablesorter.min.js');
      }

      $form['syncronized_connections'] = array(
        '#type' => 'item',
        '#title' => '<h3 class="semantic-connector-table-title">' . t('Already synchronized Drupal taxonomies') . '</h3>',
        '#markup' => drupal_render($table),
      );
    }

    // Potential synchronization candidates.
    if (!empty($rows)) {
      $form['potential_connections'] = array(
        '#type' => 'fieldset',
        '#title' => t('Potential interconnection between the Drupal taxonomies and the %rootlevel', array('%rootlevel' => ($settings['root_level'] == 'project' ? t('PoolParty project') : t('PoolParty concept scheme')))),
        '#collapsible' => TRUE,
        '#collapsed' => !empty($connected_rows),
      );

      // Create the table for the connections.
      $table = array();
      $table['connections'] = array(
        '#theme' => 'table',
        '#header' => array(
          t('Drupal taxonomy'),
          t('Operations'),
          $settings['root_level'] == 'project' ? t('PoolParty project') : t('PoolParty concept scheme'),
        ),
        '#rows' => $rows,
        '#attributes' => array(
          'id' => 'pp-taxonomy-manager-interconnection-table',
          'class' => array('semantic-connector-tablesorter'),
        ),
        '#attached' => array(
          'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
        ),
      );

      // Add the tablesorter library if available.
      if ($tablesorter_available) {
        $table['#attached']['js'] = array($tablesorter_path . '/jquery.tablesorter.min.js');
      }

      $form['potential_connections']['connections'] = array(
        '#type' => 'item',
        '#markup' => drupal_render($table),
      );
    }
  }

  // Save and cancel buttons.
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['delete'] = array(
    '#type' => 'button',
    '#value' => t('Delete'),
    '#attributes' => array('onclick' => 'window.location.href = "' . url('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/delete') . '"; return false;'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => (isset($_GET['destination']) ? $_GET['destination'] : 'admin/config/semantic-drupal/pp-taxonomy-manager'),
  );

  return $form;
}

/**
 * The submit handler for the PoolParty Taxonomy Manager configuration admin form.
 */
function pp_taxonomy_manager_configuration_form_submit($form, &$form_state) {
  $config = new PPTaxonomyManagerConfig($form_state['values']['tmid']);
  $config->setTitle($form_state['values']['title']);
  $config->save();

  drupal_set_message(t('PoolParty Taxonomy Manager configuration %title has been updated.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
}

/**
 * Add a PoolParty Taxonomy Manager configuration for a predefined PP connection + project.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param int $connection_id
 *   The ID of the PoolParty server connection
 * @param string $project_id
 *   The ID of the project on the PoolParty server
 *
 * @return array
 *   The Drupal form array of the confirmation form.
 */
function pp_taxonomy_manager_add_config_for_fixed_connection_form($form, &$form_state, $connection_id, $project_id) {
  $form_state['connection_id'] = $connection_id;
  $form_state['project_id'] = $project_id;

  $connection = SemanticConnector::getConnection('pp_server', $connection_id);
  $pp_config = $connection->getConfig();
  $project_title = '';
  foreach ($pp_config['projects'] as $project) {
    if ($project->id == $project_id) {
      $project_title = $project->title;
      break;
    }
  }

  if (empty($project_title)) {
    $form['description'] = array(
      '#markup' => t('The project with the ID $id does not exists.', array(
        '$id' => $project_id,
      )),
    );
    $form['cancel'] = array(
      '#type' => 'link',
      '#title' => t('Cancel'),
      '#href' => 'admin/config/semantic-drupal/semantic-connector',
      '#prefix' => '<div class="admin-form-submit-buttons" style="margin-top: 40px;">',
      '#suffix' => '</div>',
    );
  }
  else {
    $form['description'] = array(
      '#markup' => t('Selected PoolParty server: <b>!server</b><br />Selected project: <b>!project</b>', array(
        '!server' => $connection->getTitle(),
        '!project' => $project_title,
      )),
    );

    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Title of the new config'),
      '#maxlength' => 255,
      '#default_value' => 'PoolParty Taxonomy Manager config for ' . $connection->getTitle() . ' (' . $project_title . ')',
      '#required' => TRUE,
    );

    // Container: Level settings.
    $form['level_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Select the taxonomy root level'),
    );
    $form['level_settings']['root_level'] = array(
      '#type' => 'radios',
      '#options' => array(
        'project' => t('Every taxonomy becomes a !rootlevel.', array('!rootlevel' => '<b>' . t('PoolParty Project') . '</b>')),
        'conceptscheme' => t('Every taxonomy becomes a !rootlevel in project "%projectname".', array('!rootlevel' => '<b>' . t('Concept Scheme') . '</b>', '%projectname' => $project_title)),
      ),
      '#default_value' => 'conceptscheme',
    );
    // Save and cancel buttons.
    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Create configuration'),
      '#prefix' => '<div class="admin-form-submit-buttons" style="margin-top: 40px;">',
    );
    $form['cancel'] = array(
      '#type' => 'link',
      '#title' => t('Cancel'),
      '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager',
      '#suffix' => '</div>',
    );
  }

  return $form;
}

/**
 * Submit handler for pp_taxonomy_manager_add_config_for_fixed_connection_form().
 */
function pp_taxonomy_manager_add_config_for_fixed_connection_form_submit($form, &$form_state) {
  $connection = SemanticConnector::getConnection('pp_server', $form_state['connection_id']);
  $project_id = $form_state['project_id'];

  // Get the current list of projects from the server to make sure the project
  // still exists.
  $project_is_valid = FALSE;
  $projects = $connection->getApi('PPT')->getProjects();
  foreach ($projects as $project) {
    if ($project->id == $project_id) {
      $project_is_valid = TRUE;

      // Create a new configuration set.
      $config = new PPTaxonomyManagerConfig();
      $config->setTitle($form_state['values']['title']);
      $config->setConnectionId($connection->getId());
      if ($form_state['values']['root_level'] == 'project') {
        $config->setProjectId(NULL);
      }
      else {
        $config->setProjectId($project_id);
      }
      $settings = $config->getDefaultConfig();
      $settings['root_level'] = $form_state['values']['root_level'];
      $config->setConfig($settings);
      $tmid = $config->save();

      drupal_set_message(t('PoolParty Taxonomy Manager configuration "%title" has been created.', array('%title' => $form_state['values']['title'])));
      // Drupal Goto to forward a destination if one is available.
      if (isset($_GET['destination'])) {
        $destination = $_GET['destination'];
        unset($_GET['destination']);
        drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid, array('query' => array('destination' => $destination)));
      }
      $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $tmid;
      break;
    }
  }

  if (!$project_is_valid) {
    drupal_set_message(t('Project-ID "%projectid" was not found on PoolParty server "%ppservertitle"', array(
      '%projectid' => $project_id,
      '%ppservertitle' => $connection->getTitle(),
    )));
    $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
  }
}

/**
 * The confirmation-form for deleting a PoolParty Taxonomy Manager configuration.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   The PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array of the confirmation form.
 */
function pp_taxonomy_manager_configuration_delete_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  $form_state['config'] = $config;
  return confirm_form($form,
    t('Are you sure you want to delete the PoolParty Taxonomy Manager configuration "%title"?', array('%title' => $config->getTitle())),
    'admin/config/semantic-drupal/pp-taxonomy-manager',
    t('This action cannot be undone.'),
    t('Delete configuration'));
}

/**
 * The submit handler for deleting a PoolParty Taxonomy Manager configuration.
 */
function pp_taxonomy_manager_configuration_delete_form_submit($form, &$form_state) {
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $config->delete();

  drupal_set_message(t('PoolParty Taxonomy Manager configuration %title has been deleted.', array('%title' => $config->getTitle())));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager';
}

/**
 * The confirmation-form for the export of a taxonomy to a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_export_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the batch is running in background.
  $bid = pp_taxonomy_manager_batch_running('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/export/' . $taxonomy->vid);
  if ($bid !== FALSE) {
    drupal_goto('batch', array('query' => array('op' => 'start', 'id' => $bid)));
    exit();
  }

  $connection = $config->getConnection();
  $settings = $config->getConfig();

  // Get the project.
  $project = NULL;
  $project_names = array();
  $pp_projects = $connection->getApi('PPT')->getProjects();
  foreach ($pp_projects as $pp_project) {
    $project_names[] = $pp_project->title;
    if ($pp_project->id == $config->getProjectId()) {
      $project = $pp_project;
      break;
    }
  }
  if ($settings['root_level'] == 'conceptscheme' && is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exist.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is already connected with a concept scheme.
  $configuration = $config->getConfig();
  if (isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is already connected, please select another one.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Get the sum of all terms from the taxonomy.
  $tree = taxonomy_get_tree($taxonomy->vid);
  $count = count($tree);

  if ($settings['root_level'] == 'conceptscheme') {
    $description = t('A new concept scheme will be created in the project %project and %count terms will be inserted.', array(
      '%project' => $project->title,
      '%count' => $count,
    ));
  }
  else {
    $form['project_names'] = array(
      '#type' => 'value',
      '#value' => $project_names,
    );
    $description = t('A new PoolParty project will be created and %count concepts / concept schemes will be inserted.', array(
      '%count' => $count,
    ));
  }
  if (!module_exists('background_batch')) {
    $description .= '<br />' . t('This can take a while. Please wait until the export is finished.');
  }
  $form['description'] = array(
    '#markup' => $description,
  );
  $form['concept_scheme_title'] = array(
    '#title' => t('Title of the new %rootobject', array('%rootobject' => (($settings['root_level'] == 'conceptscheme') ? t('concept scheme') : t('project')))),
    '#type' => 'textfield',
    '#default_value' => $taxonomy->name,
    '#required' => TRUE,
  );

  // Language mapping.
  if (module_exists('i18n_taxonomy') && in_array($taxonomy->i18n_mode, array(
      I18N_MODE_LOCALIZE,
      I18N_MODE_TRANSLATE,
    ))) {
    $available_languages = language_list();
  }
  else {
    $available_languages = array(language_default());
  }
  // Map the available languages and remove disabled ones.
  $enabled_languages = array();
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $enabled_languages[$lang->language] = $lang->name;
    }
  }

  $default_language = language_default('language');
  if ($settings['root_level'] == 'project') {
    $form['default_language'] = array(
      '#type' => 'select',
      '#title' => t('Default project language'),
      '#description' => t('Select the default language of the new PoolParty project'),
      '#options' => $enabled_languages,
      '#default_value' => $default_language,
      '#required' => TRUE,
    );
  }

  $form['languages'] = array(
    '#type' => 'fieldset',
    '#title' => t('Language Mapping'),
    '#description' => t('Map the Drupal languages with the PoolParty project languages.') . (count($enabled_languages) > 1 ? '<br />' . t('The term-translations of the non-selected languages are not exported.') : ''),
    '#tree' => TRUE,
  );

  $pp_languages = $connection->getApi('PPT')->getLanguages();
  if ($settings['root_level'] == 'conceptscheme') {
    $project_language_options = array();
    foreach ($project->availableLanguages as $project_language) {
      if (isset($pp_languages[$project_language])) {
        $project_language_options[$project_language] = $pp_languages[$project_language];
      }
    }
    asort($project_language_options);

    foreach ($enabled_languages as $lang_id => $lang_title) {
      $form['languages'][$lang_id] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang_title)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => (isset($project_language_options[$lang_id]) ? $lang_id : ''),
        '#required' => ($lang_id == $default_language ? TRUE : FALSE),
      );
    }
  }
  else {
    foreach ($enabled_languages as $lang_id => $lang_title) {
      $form['languages'][$lang_id] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang_title)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $pp_languages,
        '#empty_option' => '',
        '#default_value' => isset($pp_languages[$lang_id]) ? $lang_id : NULL,
        '#required' => ($lang_id == $default_language ? TRUE : FALSE),
      );
    }
  }

  $form['terms_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('Taxonomy terms per request'),
    '#description' => t('The number of terms, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished exporting ALL your terms, what results in a shorter duration of the bulk exporting process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk exporting process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  if (module_exists('background_batch')) {
    global $user;
    $form['inform_user'] = array(
      '#type' => 'checkbox',
      '#title' => 'Send an e-mail out when the process is completed',
    );
    $form['inform_user_email_container'] = array(
      '#type' => 'container',
      '#attributes' => array('style' => array('padding-left:18px;')),
    );
    $form['inform_user_email_container']['inform_user_email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail address'),
      '#description' => t('A summary will be sent to this e-mail address when the process is completed.'),
      '#default_value' => $user->mail,
      '#states' => array(
        'visible' => array(':input[name="inform_user"]' => array('checked' => TRUE)),
      ),
    );
  } else {
    $text = t('For more usability with a large number of taxonomy terms, please install the "<a href="https://www.drupal.org/project/background_process" target="_blank">Background Process</a>" module and activate the "Background Batch" sub-module.') . '<br />';
    $text .= t('This modules takes over the existing Batch API and runs batch jobs in a background process. This means that if you leave the batch page (e.g. you close the browser), the jobs continues, and you can return to the progress indicator later.');
    $form['install_info'] = array(
      '#prefix' => '<div class="form-item">',
      '#suffix' => '</div>',
      '#markup' => $text,
    );
  }

  if (module_exists('i18n_taxonomy') && $taxonomy->i18n_mode == I18N_MODE_LOCALIZE) {
    $form['info'] = array(
      '#prefix' => '<p><label>' . t('Attention:') . '</label>',
      '#markup' => t('If multiple Drupal languages have a selected PoolParty language the translation mode of this taxonomy will be changed from "Localize" to "Translate" first before the export begins.'),
      '#suffix' => '</p>',
    );
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Export taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js');

  return $form;
}

/**
 * Validate-Handler of the form to export terms to a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_export_form_validate($form, &$form_state) {
  // Ignore empty languages.
  if (!empty($form_state['values']['languages'])) {
    $form_state['values']['languages'] = array_filter($form_state['values']['languages']);
  }

  $values = $form_state['values'];

  if (!empty($values['project_names']) && in_array($values['concept_scheme_title'], $values['project_names'])) {
    form_set_error('concept_scheme_title', t('A PoolParty project with the title "%title" already exists. Please select an other title.', array('%title' => $values['concept_scheme_title'])));
  }

  // Check whether all languages are different.
  $languages = array_unique($values['languages']);
  if (count($values['languages']) != count($languages)) {
    form_set_error('languages', t('The selected languages must be different.'));
  }

  $concepts_per_request = $values['terms_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('terms_per_request', t('Only values in the range of 1 - 100 are allowed for field "Taxonomy terms per request"'));
  }

  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email']) && !valid_email_address($values['inform_user_email'])) {
    form_set_error('inform_user_email', t('Please enter a valid email address.'));
  }
}

/**
 * The submit handler for the export confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_export_form_submit($form, &$form_state) {

  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];
  $settings = $config->getConfig();

  // Get the data properties for the data pushing process.
  $properties = PPTaxonomyManager::getTaxonomyFieldProperties();
  $data_properties = array_keys($properties);

  $terms_per_request = $form_state['values']['terms_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Add URI and alt. labels fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  if ($settings['root_level'] == 'conceptscheme') {
    // Create the new concept scheme in the PoolParty thesaurus.
    $root_uri = $manager->createConceptScheme($taxonomy, $values['concept_scheme_title']);
  }
  else {
    $root_uri = $manager->createProject($taxonomy, $values['concept_scheme_title'], $values['default_language'], array_values($values['languages']));
  }

  if ($root_uri !== FALSE) {
    // Connect the taxonomy with the new concept scheme / project.
    $config->addConnection($taxonomy->vid, $root_uri, $languages, $data_properties);

    // Collect the email data for the summery email.
    $inform_user = array();
    if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email'])) {
      $inform_user = array(
        'user_email' => $values['inform_user_email'],
        'batch_url' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/export/' . $taxonomy->vid,
      );
    }

    // Export all taxonomy terms.
    $manager->exportTaxonomyTerms($taxonomy, $root_uri, $languages, $terms_per_request, $inform_user);
  }
  // The project could not be created.
  else {
    drupal_set_message(t('There was an error during the creation of the project.'), 'error');
  }

  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the config of a taxonomy to a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_sync_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the batch is running in background.
  $bid = pp_taxonomy_manager_batch_running('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/sync/' . $taxonomy->vid);
  if ($bid !== FALSE) {
    drupal_goto('batch', array('query' => array('op' => 'start', 'id' => $bid)));
    exit();
  }

  $connection = $config->getConnection();
  $settings = $config->getConfig();

  // Get the ID of the project.
  if ($settings['root_level'] == 'conceptscheme') {
    $project_id = $config->getProjectId();
  }
  else {
    if (!isset($settings['taxonomies'][$taxonomy->vid])) {
      drupal_set_message(t('The selected taxonomy is not yet connected to PoolParty.'), 'error');
      drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
      exit();
    }
    $project_id = $settings['taxonomies'][$taxonomy->vid];
  }

  // Get the project.
  $project = NULL;
  $pp_projects = $connection->getApi('PPT')->getProjects();
  foreach ($pp_projects as $pp_project) {
    if ($pp_project->id == $project_id) {
      $project = $pp_project;
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exist.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is connected with a concept scheme.
  if (!isset($settings['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is not connected, please export the taxonomy first.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $description = t('The taxonomy %taxonomy will be updated. It means that terms will be updated, deleted and/or created.', array('%taxonomy' => $taxonomy->name));
  $description .= '<br />' . t('This can take a while. Please wait until the synchronization is finished.');

  $form['description'] = array(
    '#markup' => $description,
  );

  // Language mapping.
  $available_languages = language_list();
  // Map the available languages and remove disabled ones.
  $enabled_languages = array();
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $enabled_languages[$lang->language] = $lang->name;
    }
  }
  $default_language = language_default('language');

  $form['languages'] = array(
    '#type' => 'fieldset',
    '#title' => t('Language Mapping'),
    '#description' => t('Map the Drupal languages with the PoolParty project languages.'),
    '#tree' => TRUE,
  );

  $pp_languages = $connection->getApi('PPT')->getLanguages();
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    if (isset($pp_languages[$project_language])) {
      $project_language_options[$project_language] = $pp_languages[$project_language];
    }
  }
  asort($project_language_options);
  if ($settings['root_level'] == 'conceptscheme') {
    foreach ($enabled_languages as $lang_id => $lang_title) {
      $form['languages'][$lang_id] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang_title)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => isset($settings['languages'][$taxonomy->vid][$lang_id]) ? $settings['languages'][$taxonomy->vid][$lang_id] : '',
        '#required' => ($lang_id == $default_language ? TRUE : FALSE),
        '#disabled' => ($lang_id == $default_language ? TRUE : FALSE),
      );
    }
  }
  else {
    foreach ($enabled_languages as $lang_id => $lang_title) {
      $form['languages'][$lang_id] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang_title)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => isset($settings['languages'][$taxonomy->vid][$lang_id]) ? $settings['languages'][$taxonomy->vid][$lang_id] : '',
        '#required' => ($lang_id == $default_language ? TRUE : FALSE),
      );
    }
  }

  $default_values = isset($settings['data_properties'][$taxonomy->vid]) ? $settings['data_properties'][$taxonomy->vid] : [];
  pp_taxonomy_manager_data_properties_form($form, $default_values);

  $form['concepts_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('PoolParty concepts per request'),
    '#description' => t('The number of concepts, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished updating ALL your concepts, what results in a shorter duration of the bulk updating process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk updating process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  if (module_exists('background_batch')) {
    global $user;
    $form['inform_user'] = array(
      '#type' => 'checkbox',
      '#title' => 'Send an e-mail out when the process is completed',
    );
    $form['inform_user_email_container'] = array(
      '#type' => 'container',
      '#attributes' => array('style' => array('padding-left:18px;')),
    );
    $form['inform_user_email_container']['inform_user_email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail address'),
      '#description' => t('A summary will be sent to this e-mail address when the process is completed.'),
      '#default_value' => $user->mail,
      '#states' => array(
        'visible' => array(':input[name="inform_user"]' => array('checked' => TRUE)),
      ),
    );
  } else {
    $text = t('For more usability with a large number of PoolParty concepts, please install the "<a href="https://www.drupal.org/project/background_process" target="_blank">Background Process</a>" module and activate the "Background Batch" sub-module.') . '<br />';
    $text .= t('This modules takes over the existing Batch API and runs batch jobs in a background process. This means that if you leave the batch page (e.g. you close the browser), the jobs continues, and you can return to the progress indicator later.');
    $form['install_info'] = array(
      '#prefix' => '<div class="form-item">',
      '#suffix' => '</div>',
      '#markup' => $text,
    );
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Synchronize taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js');

  return $form;
}

/**
 * Validate-Handler of the form to synchronize concepts with a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_sync_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  $default_language = language_default('language');
  if (!module_exists('i18n_taxonomy')) {
    foreach ($values['languages'] as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang) && $drupal_lang != LANGUAGE_NONE && $drupal_lang != $default_language) {
        $drupal_languages = language_list();
        form_set_error('languages][' . $drupal_lang, t('Language "%language" requires translation of taxonomies as it is not your default site language.<br /> Install and enable module "!internationalizationlink" and its sub-module "Taxonomy translation" to make multilingual tagging possible.', array(
          '%language' => $drupal_languages[$drupal_lang]->name,
          '!internationalizationlink' => l(t('Internationalization'), 'https://www.drupal.org/project/i18n', array('attributes' => array('target' => '_blank'))),
        )));
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('concepts_per_request', t('Only values in the range of 1 - 100 are allowed for field "PoolParty concepts per request"'));
  }

  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email']) && !valid_email_address($values['inform_user_email'])) {
    form_set_error('inform_user_email', t('Please enter a valid email address.'));
  }
}

/**
 * The submit handler for the synchronization confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_sync_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];

  // Get the data properties for the data fetching process.
  $data_properties = [];
  if (isset($values['data_properties'])) {
    foreach ($values['data_properties'] as $property) {
      if ($property) {
        $data_properties[] = $property;
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Add URI and alt. label fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Update the connection.
  $settings = $config->getConfig();
  $root_uri = $settings['taxonomies'][$taxonomy->vid];
  $config->updateConnection($taxonomy->vid, $root_uri, $languages, $data_properties);

  // Collect the email data for the summery email.
  $inform_user = array();
  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email'])) {
    $inform_user = array(
      'user_email' => $values['inform_user_email'],
      'batch_url' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/sync/' . $taxonomy->vid,
    );
  }

  // Update all taxonomy terms.
  try {
    $manager->updateTaxonomyTerms('sync', $taxonomy, $root_uri, $languages, $data_properties, FALSE, $concepts_per_request, $inform_user);
  } catch (Exception $e) {
    drupal_set_message($e->getMessage(), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the conversion of a PowerTagging taxonomy to a
 * completed one.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $powertagging_config
 *   The PowerTagging configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_powertagging_taxonomy_update_form($form, &$form_state, PPTaxonomyManagerConfig $config, $powertagging_config) {
  // Check if PowerTagging is enabled and the config exists.
  if (!is_object($powertagging_config)) {
    drupal_set_message(t('The PowerTagging configuration does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if taxonomy exists.
  $taxonomy = taxonomy_vocabulary_load($powertagging_config->config['projects'][$powertagging_config->project_id]['taxonomy_id']);
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the batch is running in background.
  $bid = pp_taxonomy_manager_batch_running('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/powertagging-taxonomy-update/' . $powertagging_config->powertagging_id);
  if ($bid !== FALSE) {
    drupal_goto('batch', array('query' => array('op' => 'start', 'id' => $bid)));
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();

  // Get the project.
  $project = NULL;
  $pp_projects = $connection->getApi('PPT')->getProjects();
  foreach ($pp_projects as $pp_project) {
    if ($pp_project->id == $powertagging_config->project_id) {
      $project = $pp_project;
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exist.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is connected with a concept scheme.
  $configuration = $config->getConfig();
  if (isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is already a completed taxonomy, please use the normal syncronization process.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $description = t('The taxonomy %taxonomy will be converted into a completed taxonomy. It means that terms will be updated, deleted and/or created and hierarchy will be added.', array('%taxonomy' => $taxonomy->name));
  $description .= '<br />' . t('This can take a while. Please wait until the conversion process is finished.');

  $form['description'] = array(
    '#markup' => $description,
  );

  // Language mapping.
  $available_languages = language_list();
  $default_language = language_default('language');

  $form['languages'] = array(
    '#type' => 'item',
    '#title' => t('Map the Drupal languages with the PoolParty project languages'),
    '#tree' => TRUE,
  );

  $pp_languages = $connection->getApi('PPT')->getLanguages();
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    if (isset($pp_languages[$project_language])) {
      $project_language_options[$project_language] = $pp_languages[$project_language];
    }
  }
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $form['languages'][$lang->language] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang->name)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => isset($project_language_options[$lang->language]) ? $lang->language : '',
        '#required' => ($lang->language == $default_language ? TRUE : FALSE),
      );
    }
  }

  pp_taxonomy_manager_data_properties_form($form, []);

  $form['concepts_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('PoolParty concepts per request'),
    '#description' => t('The number of concepts, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished updating ALL your concepts, what results in a shorter duration of the bulk updating process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk updating process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  if (module_exists('background_batch')) {
    global $user;
    $form['inform_user'] = array(
      '#type' => 'checkbox',
      '#title' => 'Send an e-mail out when the process is completed',
    );
    $form['inform_user_email_container'] = array(
      '#type' => 'container',
      '#attributes' => array('style' => array('padding-left:18px;')),
    );
    $form['inform_user_email_container']['inform_user_email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail address'),
      '#description' => t('A summary will be sent to this e-mail address when the process is completed.'),
      '#default_value' => $user->mail,
      '#states' => array(
        'visible' => array(':input[name="inform_user"]' => array('checked' => TRUE)),
      ),
    );
  } else {
    $text = t('For more usability with a large number of PoolParty concepts, please install the "<a href="https://www.drupal.org/project/background_process" target="_blank">Background Process</a>" module and activate the "Background Batch" sub-module.') . '<br />';
    $text .= t('This modules takes over the existing Batch API and runs batch jobs in a background process. This means that if you leave the batch page (e.g. you close the browser), the jobs continues, and you can return to the progress indicator later.');
    $form['install_info'] = array(
      '#prefix' => '<div class="form-item">',
      '#suffix' => '</div>',
      '#markup' => $text,
    );
  }

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Make complete taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;
  $form_state['powertagging_config'] = $powertagging_config;

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js');

  return $form;
}

/**
 * Validate-Handler of the form to convert a PowerTagging taxonomy.
 */
function pp_taxonomy_manager_taxonomy_powertagging_taxonomy_update_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  $default_language = language_default('language');
  if (!module_exists('i18n_taxonomy')) {
    foreach ($values['languages'] as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang) && $drupal_lang != LANGUAGE_NONE && $drupal_lang != $default_language) {
        $drupal_languages = language_list();
        form_set_error('languages][' . $drupal_lang, t('Language "%language" requires translation of taxonomies as it is not your default site language.<br /> Install and enable module "!internationalizationlink" and its sub-module "Taxonomy translation" to make multilingual tagging possible.', array(
          '%language' => $drupal_languages[$drupal_lang]->name,
          '!internationalizationlink' => l(t('Internationalization'), 'https://www.drupal.org/project/i18n', array('attributes' => array('target' => '_blank'))),
        )));
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('concepts_per_request', t('Only values in the range of 1 - 100 are allowed for field "PoolParty concepts per request"'));
  }

  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email']) && !valid_email_address($values['inform_user_email'])) {
    form_set_error('inform_user_email', t('Please enter a valid email address.'));
  }
}

/**
 * The submit handler of the form to convert a PowerTagging taxonomy.
 */
function pp_taxonomy_manager_taxonomy_powertagging_taxonomy_update_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];
  $powertagging_config = $form_state['powertagging_config'];

  // Get the data properties for the data fetching process.
  $data_properties = [];
  if (isset($values['data_properties'])) {
    foreach ($values['data_properties'] as $property) {
      if ($property) {
        $data_properties[] = $property;
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);

  $manager = new PPTaxonomyManager($config);

  // Add URI and alt. label fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Update the connection.
  $config->addConnection($taxonomy->vid, $powertagging_config->project_id, $languages, $data_properties);

  // Collect the email data for the summery email.
  $inform_user = array();
  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email'])) {
    $inform_user = array(
      'user_email' => $values['inform_user_email'],
      'batch_url' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/powertagging-taxonomy-update/' . $powertagging_config->powertagging_id,
    );
  }

  // Update all taxonomy terms.
  try {
    $manager->updateTaxonomyTerms('powertagging_taxonomy_update', $taxonomy, $powertagging_config->project_id, $languages, $data_properties, TRUE, $concepts_per_request, $inform_user);
  } catch (Exception $e) {
    drupal_set_message($e->getMessage(), 'error');
  }
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the import of a taxonomy from a PoolParty server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_import_form($form, &$form_state, PPTaxonomyManagerConfig $config) {
  $settings = $config->getConfig();
  $root_uri = str_replace('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/', '', current_path());

  // Check if the batch is running in background.
  $bid = pp_taxonomy_manager_batch_running('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $root_uri);
  if ($bid !== FALSE) {
    drupal_goto('batch', array('query' => array('op' => 'start', 'id' => $bid)));
    exit();
  }

  // Get the project.
  $connection = $config->getConnection();
  $pp_projects = $connection->getApi('PPT')->getProjects();
  $project = NULL;
  foreach ($pp_projects as $pp_project) {
    if ($pp_project->id == ($settings['root_level'] == 'project' ? $root_uri : $config->getProjectId())) {
      $project = $pp_project;
      break;
    }
  }
  if (is_null($project)) {
    drupal_set_message(t('The configured PoolParty project does not exist.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $root_object = NULL;
  if ($settings['root_level'] == 'conceptscheme') {
    // Check if concept scheme URI is given and is a url.
    if (!valid_url($root_uri, TRUE)) {
      drupal_set_message(t('The URI from the selected concept scheme is not valid.'), 'error');
      drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
      exit();
    }

    // Check if concept scheme exists.
    $concept_schemes = $config->getConnection()
      ->getApi('PPT')
      ->getConceptSchemes($config->getProjectId());
    foreach ($concept_schemes as $scheme) {
      if ($scheme->uri == $root_uri) {
        $root_object = $scheme;
        break;
      }
    }

    if (is_null($root_object)) {
      drupal_set_message(t('The selected concept scheme does not exists.'), 'error');
      drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
      exit();
    }
  }
  else {
    $root_object = $project;
  }

  // Check if the taxonomy is already connected with a concept scheme.
  if (in_array($root_uri, $settings['taxonomies'])) {
    drupal_set_message(t('The %rootobject is already connected, please select another one.', array('%rootobject' => (($settings['root_level'] == 'conceptscheme') ? t('concept scheme') : t('project')) . ' ' . $root_object->title)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the new taxonomy already exists in Drupal.
  $machine_name = PPTaxonomyManager::machineName($root_object->title);
  $taxonomy_name = db_select('taxonomy_vocabulary', 't')
    ->fields('t', array('name'))
    ->condition('machine_name', $machine_name)
    ->execute()
    ->fetchField();

  if ($settings['root_level'] == 'conceptscheme') {
    $description = t('A new taxonomy will be created and all concepts from the concept scheme %scheme will be inserted as terms.', array('%scheme' => $root_object->title));
  }
  else {
    $description = t('A new taxonomy will be created and all concepts / concept schemes from the project %project will be inserted as terms.', array('%project' => $root_object->title));
  }
  $description .= '<br />' . t('This can take a while. Please wait until the import is finished.');
  $form['description'] = array(
    '#markup' => $description,
  );
  $field_description = t('Please enter a name of a taxonomy, which does not yet exist.');
  if (!empty($taxonomy_name)) {
    $field_description .= '<br />' . t('The taxonomy %taxonomy (machine name: %machine_name) already exists. Its terms will be updated, deleted and/or created.', array(
        '%taxonomy' => $taxonomy_name,
        '%machine_name' => $machine_name,
      ));
  }
  $form['taxonomy_name'] = array(
    '#title' => t('Name of the new taxonomy'),
    '#type' => 'textfield',
    '#default_value' => $root_object->title,
    '#description' => $field_description,
    '#required' => TRUE,
  );

  // Language mapping.
  $available_languages = language_list();
  $default_language = language_default('language');
  $pp_languages = $connection->getApi('PPT')->getLanguages();
  $project_language_options = array();
  foreach ($project->availableLanguages as $project_language) {
    if (isset($pp_languages[$project_language])) {
      $project_language_options[$project_language] = $pp_languages[$project_language];
    }
  }
  $form['languages'] = array(
    '#type' => 'item',
    '#title' => t('Map the Drupal languages with the PoolParty project languages'),
    '#tree' => TRUE,
  );
  foreach ($available_languages as $lang) {
    if ($lang->enabled) {
      $form['languages'][$lang->language] = array(
        '#type' => 'select',
        '#title' => t('Drupal language %language', array('%language' => $lang->name)),
        '#description' => t('Select the PoolParty project language'),
        '#options' => $project_language_options,
        '#empty_option' => '',
        '#default_value' => ($lang->language == $default_language ? $project->defaultLanguage : ''),
        '#required' => ($lang->language == $default_language ? TRUE : FALSE),
      );
    }
  }

  pp_taxonomy_manager_data_properties_form($form, ['skos:altLabel', 'skos:hiddenLabel']);

  $form['preserve_concepts'] = array(
    '#type' => 'checkbox',
    '#title' => t('Preserve old concepts'),
    '#description' => t('If the concepts get imported into an existing taxonomy, taxonomy terms which can\'t be matched by URI or label normally get deleted') . '<br />' . t(' By checking "Preserve old concepts" instead of deleting them they get moved under a special concept named "Freeterms" without a URI.'),
    '#default_value' => FALSE,
  );

  $form['concepts_per_request'] = array(
    '#type' => 'textfield',
    '#title' => t('PoolParty concepts per request'),
    '#description' => t('The number of concepts, that get processed during one HTTP request. (Allowed value range: 1 - 100)') . '<br />' . t('The higher this number is, the less HTTP requests have to be sent to the server until the batch finished updating ALL your concepts, what results in a shorter duration of the bulk updating process.') . '<br />' . t('Numbers too high can result in a timeout, which will break the whole bulk updating process.'),
    '#required' => TRUE,
    '#default_value' => 10,
  );

  if (module_exists('background_batch')) {
    global $user;
    $form['inform_user'] = array(
      '#type' => 'checkbox',
      '#title' => 'Send an e-mail out when the process is completed',
    );
    $form['inform_user_email_container'] = array(
      '#type' => 'container',
      '#attributes' => array('style' => array('padding-left:18px;')),
    );
    $form['inform_user_email_container']['inform_user_email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail address'),
      '#description' => t('A summary will be sent to this e-mail address when the process is completed.'),
      '#default_value' => $user->mail,
      '#states' => array(
        'visible' => array(':input[name="inform_user"]' => array('checked' => TRUE)),
      ),
    );
  } else {
    $text = t('For more usability with a large number of PoolParty concepts, please install the "<a href="https://www.drupal.org/project/background_process" target="_blank">Background Process</a>" module and activate the "Background Batch" sub-module.') . '<br />';
    $text .= t('This modules takes over the existing Batch API and runs batch jobs in a background process. This means that if you leave the batch page (e.g. you close the browser), the jobs continues, and you can return to the progress indicator later.');
    $form['install_info'] = array(
      '#prefix' => '<div class="form-item">',
      '#suffix' => '</div>',
      '#markup' => $text,
    );
  }

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => t('Import taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['root_object'] = $root_object;

  // Add JavaScript
  drupal_add_js(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js');

  return $form;
}

/**
 * Validate-Handler of the form to import concepts from a PoolParty server.
 */
function pp_taxonomy_manager_taxonomy_import_form_validate($form, &$form_state) {
  $values = $form_state['values'];

  $default_language = language_default('language');
  if (!module_exists('i18n_taxonomy')) {
    foreach ($values['languages'] as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang) && $drupal_lang != LANGUAGE_NONE && $drupal_lang != $default_language) {
        $drupal_languages = language_list();
        form_set_error('languages][' . $drupal_lang, t('Language "%language" requires translation of taxonomies as it is not your default site language.<br /> Install and enable module "!internationalizationlink" and its sub-module "Taxonomy translation" to make multilingual tagging possible.', array(
          '%language' => $drupal_languages[$drupal_lang]->name,
          '!internationalizationlink' => l(t('Internationalization'), 'https://www.drupal.org/project/i18n', array('attributes' => array('target' => '_blank'))),
        )));
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  if (empty($concepts_per_request) || !ctype_digit($concepts_per_request) || (int) $concepts_per_request == 0 || (int) $concepts_per_request > 100) {
    form_set_error('concepts_per_request', t('Only values in the range of 1 - 100 are allowed for field "PoolParty concepts per request"'));
  }

  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email']) && !valid_email_address($values['inform_user_email'])) {
    form_set_error('inform_user_email', t('Please enter a valid email address.'));
  }
}

/**
 * The submit handler for the import confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_import_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $settings = $config->getConfig();
  $root_object = $form_state['root_object'];
  $root_uri = ($settings['root_level'] == 'project') ? $root_object->id : $root_object->uri;

  // Get the data properties for the data fetching process.
  $data_properties = [];
  if (isset($values['data_properties'])) {
    foreach ($values['data_properties'] as $property) {
      if ($property) {
        $data_properties[] = $property;
      }
    }
  }

  $concepts_per_request = $values['concepts_per_request'];
  $languages = pp_taxonomy_manager_order_languages($values['languages']);
  $preserve_concepts = $values['preserve_concepts'];

  $manager = new PPTaxonomyManager($config);

  // Create the new taxonomy and set the translation mode.
  $taxonomy = $manager->createTaxonomy($root_object, $values['taxonomy_name']);
  $manager->setTranslationMode($taxonomy, $languages);

  // Add URI and alt. labels fields (if not exists) to the taxonomy.
  $manager->adaptTaxonomyFields($taxonomy);

  // Connect the new taxonomy with the concept scheme.
  $config->addConnection($taxonomy->vid, $root_uri, $languages, $data_properties);

  // Collect the email data for the summery email.
  $inform_user = array();
  if (isset($values['inform_user']) && !empty($values['inform_user']) && !empty($values['inform_user_email'])) {
    $inform_user = array(
      'user_email' => $values['inform_user_email'],
      'batch_url' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId() . '/import/' . $root_uri,
    );
  }

  // Import all concepts.
  try {
    $manager->updateTaxonomyTerms('import', $taxonomy, $root_uri, $languages, $data_properties, $preserve_concepts, $concepts_per_request, $inform_user);
  } catch (Exception $e) {
    drupal_set_message($e->getMessage(), 'error');
  }

  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * The confirmation-form for the disconnection from a taxonomy to a PP server.
 *
 * @param array $form
 *   The form array.
 * @param array &$form_state
 *   The form_state array.
 * @param PPTaxonomyManagerConfig $config
 *   An instance of a PoolParty Taxonomy Manager configuration.
 * @param object $taxonomy
 *   A taxonomy object.
 *
 * @return array
 *   The Drupal form array.
 */
function pp_taxonomy_manager_taxonomy_disconnect_form($form, &$form_state, PPTaxonomyManagerConfig $config, $taxonomy) {
  // Check if taxonomy exists.
  if ($taxonomy === FALSE) {
    drupal_set_message(t('The selected taxonomy does not exists.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $settings = $config->getConfig();
  if ($settings['root_level'] == 'project' && !isset($settings['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The selected taxonomy is not yet connected to PoolParty.'), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  // Check if the taxonomy is connected with a concept scheme.
  $configuration = $config->getConfig();
  if (!isset($configuration['taxonomies'][$taxonomy->vid])) {
    drupal_set_message(t('The taxonomy %taxonomy is already disconnected, please select another one.', array('%taxonomy' => $taxonomy->name)), 'error');
    drupal_goto('admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId());
    exit();
  }

  $description = t('The connection between the taxonomy %taxonomy and the PoolParty server will be deleted.', array('%taxonomy' => $taxonomy->name));
  $description .= '<br />' . t('Please wait until the disconnection is finished.');

  $form['description'] = array(
    '#prefix' => '<p>',
    '#markup' => $description,
    '#suffix' => '</p>',
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Disconnect taxonomy'),
  );
  $form['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId(),
    '#suffix' => '</div>',
  );

  $form_state['config'] = $config;
  $form_state['taxonomy'] = $taxonomy;

  return $form;
}

/**
 * The submit handler for the disconnect confirmation-form.
 */
function pp_taxonomy_manager_taxonomy_disconnect_form_submit($form, &$form_state) {
  /* @var $config PPTaxonomyManagerConfig */
  $config = $form_state['config'];
  $taxonomy = $form_state['taxonomy'];

  // Delete the connection.
  $config->deleteConnection($taxonomy->vid);

  // Delete all the logs and hash data.
  $manager = new PPTaxonomyManager($config);
  $manager->deleteSyncData($taxonomy->vid);

  $connection = $config->getConnection();
  drupal_set_message(t('The connection between the Drupal taxonomy %taxonomy and the PoolParty server %server has been deleted successfully.', array(
    '%taxonomy' => $taxonomy->name,
    '%server' => $connection->getTitle(),
  )));
  $form_state['redirect'] = 'admin/config/semantic-drupal/pp-taxonomy-manager/' . $config->getId();
}

/**
 * It extends an existing form with the data properties.
 * @param array $form
 *   The form array.
 * @param array $default_values
 *   The default values.
 */
function pp_taxonomy_manager_data_properties_form(&$form, $default_values) {
  $properties = PPTaxonomyManager::getTaxonomyFieldProperties();
  $form['data_properties'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Select the properties that will be saved in addition to the taxonomy terms'),
    '#description' => t('The data for the unselected properties will be deleted from the Drupal taxonomy.'),
    '#options' => $properties,
    '#default_value' => $default_values,
  );
}

/**
 * Show a list of PowerTagging configurations to use for concept suggestion.
 *
 * @return array
 *   A renderable array of the list.
 */
function pp_taxonomy_manager_suggest_concepts_config_list() {
  $output = array();

  $output['powertagging_title'] = array(
    '#type' => 'markup',
    '#markup' => '<h3 class="semantic-connector-table-title">' . t('Suggest concepts for a PowerTagging configuration') . '</h3><p class="description">' . t('This area allows you to suggest extracted free-terms (saved as Drupal taxonomy terms) as PoolParty concepts.') . '</p>',
  );

  $config_sets = powertagging_config_load();
  $rows = array();
  foreach ($config_sets as $config) {
    /** @var \SemanticConnectorPPTApi $ppt_api */
    $ppt_api = $config->connection->getApi('PPT');

    $field_instances = powertagging_get_tagging_field_instances($config->powertagging_id);
    $fields = powertagging_get_tagging_field_options_list($field_instances);
    $fields = empty($fields) ? '<div class="semantic-connector-italic">' . t('not yet set') . '</div>' : theme('item_list', array('items' => $fields));

    // Count the free terms of the PowerTagging configuration.
    $field_names = [];
    foreach ($field_instances as $field_instance) {
      $field_names[] = $field_instance['field_name'];
    }

    // Get previously suggested concepts.
    $suggested_concepts = $ppt_api->getSuggestedConcepts($config->project_id);
    $suggested_concepts_labels = [];
    foreach ($suggested_concepts as $suggested_concept) {
      $suggested_concepts_labels[] = $suggested_concept['prefLabels'][0]['label'];
    }

    $freeterm_count = 0;
    if (!empty($field_names)) {
      $field_name_query = NULL;
      foreach ($field_names as $field_name) {
        $current_field_query = db_select('field_data_' . $field_name, 'fd');
        $current_field_query->addField('fd', $field_name . '_tid', 'tid');
        if (!is_null($field_name_query)) {
          $current_field_query->union($field_name_query);
        }
        $field_name_query = $current_field_query;
      }

      $term_query = db_select('taxonomy_term_data', 't')
        ->fields('t', array('tid'));
      $term_query->distinct();

      if (!empty($suggested_concepts_labels)) {
        $term_query->condition('t.name', $suggested_concepts_labels, 'NOT IN');
      }

      $term_query->addJoin('', $field_name_query, 'fd', 't.tid = fd.tid');
      $term_query->leftJoin('field_data_field_uri', 'u', 'u.entity_type = \'taxonomy_term\' AND t.tid = u.entity_id');

      $term_query->isNull('u.field_uri_value');

      $freeterm_count = intval($term_query->countQuery()
        ->execute()
        ->fetchField());
    }

    $actions = [];
    if ($freeterm_count > 0) {
      $actions[] = l(t('Suggest Concepts'), 'admin/config/semantic-drupal/pp-taxonomy-manager/suggest-concepts/' . $config->powertagging_id);
    }

    $title = '<div class="semantic-connector-led" data-server-id="' . $config->connection->getId() . '" data-server-type="pp-server" title="' . t('Checking service') . '"></div>';
    $title .= $config->title;
    $rows[] = array(
      $title,
      l($config->connection->getTitle(), $config->connection->getUrl() . '/PoolParty', array('attributes' => array('target' => '_blank'))),
      (isset($config->config['projects'][$config->project_id]) ? $config->config['projects'][$config->project_id]['label'] : '<i>' . t('project label not found') . '</i>'),
      $fields,
      $freeterm_count,
      count($suggested_concepts),
      implode(' | ', $actions),
    );
  }

  $output['powertagging'] = array(
    '#theme' => 'table',
    '#header' => array(
      t('PowerTagging configuration'),
      t('PoolParty server'),
      t('Selected project'),
      t('Available in entity type'),
      t('Number of free-terms'),
      t('Number of pending suggestions'),
      t('Operations'),
    ),
    '#rows' => $rows,
    '#attributes' => array(
      'id' => 'pp-taxonomy-manager-suggest-concept-config-list-table',
      'class' => array('semantic-connector-tablesorter'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
    ),
  );

  // Add the tablesorter library if available.
  $tablesorter_path = 'sites/all/libraries/tablesorter';
  if (module_exists('libraries')) {
    $tablesorter_path = libraries_get_path('tablesorter');
  }
  if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
    $output['#attached']['js'] = array(libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js');
  }

  return $output;
}

/**
 * Suggest Drupal taxonomy terms as PoolParty concepts.
 */
function pp_taxonomy_manager_suggest_concepts_form($form, &$form_state, $powertagging_config) {
  $form['powertagging_config'] = array(
    '#type' => 'value',
    '#value' => $powertagging_config,
  );

  /** @var \SemanticConnectorPPTApi $ppt_api */
  $ppt_api = $powertagging_config->connection->getApi('PPT');

  // Get previously suggested concepts.
  $suggested_concepts = $ppt_api->getSuggestedConcepts($powertagging_config->project_id);
  $suggested_concepts_labels = [];
  foreach ($suggested_concepts as $suggested_concept) {
    $suggested_concepts_labels[] = $suggested_concept['prefLabels'][0]['label'];
  }

  $field_instances = powertagging_get_tagging_field_instances($powertagging_config->powertagging_id);
  $field_names = [];
  foreach ($field_instances as $field_instance) {
    $field_names[$field_instance['field_name']] = $field_instance['label'];
  }

  $field_name_query = NULL;
  foreach (array_keys($field_names) as $field_name) {
    if (isset($form_state['values']['field_filter'])) {
      $selected_fields = array_filter($form_state['values']['field_filter']);
      if (!empty($selected_fields) && !in_array($field_name, $selected_fields)) {
        continue;
      }
    }

    $current_field_query = db_select('field_data_' . $field_name, 'fd');
    $current_field_query->addField('fd', $field_name . '_tid', 'tid');
    if (!is_null($field_name_query)) {
      $current_field_query->union($field_name_query);
    }
    $field_name_query = $current_field_query;
  }

  $term_query = db_select('taxonomy_term_data', 't')
    ->fields('t', array('tid', 'name'));
  if (isset($form_state['values']['text_filter']) && !empty($form_state['values']['text_filter'])) {
    $term_query->condition('t.name', '%' . db_like($form_state['values']['text_filter']) . '%', 'LIKE');
  }
  if (!empty($suggested_concepts_labels)) {
    $term_query->condition('t.name', $suggested_concepts_labels, 'NOT IN');
  }

  $term_query->addExpression('COUNT(t.tid)', 'count');
  $term_query->addJoin('', $field_name_query, 'fd', 't.tid = fd.tid');
  $term_query->leftJoin('field_data_field_uri', 'u', 'u.entity_type = \'taxonomy_term\' AND t.tid = u.entity_id');

  $term_query->isNull('u.field_uri_value');
  $term_query->groupBy('t.tid');
  $term_query->groupBy('t.name');
  $term_query->orderBy('count', 'DESC');
  $term_query->orderBy('t.name', 'ASC');
  $term_query->range(0, 25);

  $terms = $term_query->execute()
    ->fetchAllAssoc('tid');

  $term_options = [];
  if (!empty($terms)) {
    $form['term_checkboxes'] = array(
      '#tree' => TRUE,
      );
    foreach ($terms as $term) {
      $term_options[$term->tid] = [
        'name' => l($term->name, 'taxonomy/term/' . $term->tid . '/edit'),
        'count' => $term->count,
      ];
    }
  }

  if (!empty($term_options)) {
    if (count($field_names) > 1) {
      $form['field_filter'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Filter by field'),
        '#options' => $field_names,
        '#ajax' => array(
          'callback' => 'pp_taxonomy_manager_suggest_concept_list_update',
          'wrapper' => 'freeterm-list-wrapper',
          'method' => 'replace',
        ),
      );
    }

    $form['text_filter'] = array(
      '#type' => 'textfield',
      '#title' => t('Search for free-term'),
      '#ajax' => array(
        'callback' => 'pp_taxonomy_manager_suggest_concept_list_update',
        'event' => 'change',
        'wrapper' => 'freeterm-list-wrapper',
        'method' => 'replace',
        'keypress'  =>  TRUE,
      ),
    );
  }

  $form['freeterm_list'] = array(
    '#type' => 'tableselect',
    '#prefix' => '<div id="freeterm-list-wrapper">',
    '#suffix' => '</div>',
    '#header' => array(
      'name' => t('Name'),
      'count' => t('Times used'),
      //'operations' => t('Operations'),
    ),
    '#options' => $term_options,
    '#empty' => t('There are no free-terms available for this PowerTagging configuration.'),
    '#attributes' => array(
      'class' => array('pp-taxonomy-manager-suggest-concept-table'),
    ),
    '#js_select' => TRUE,
  );

  $form['#attached'] = array(
    'js' => array(drupal_get_path('module', 'pp_taxonomy_manager') . '/js/pp_taxonomy_manager.admin.js'),
  );

  // Save and cancel buttons.
  $form['suggest'] = array(
    '#type' => 'submit',
    '#value' => t('Suggest'),
  );

  if (!empty($suggested_concepts)) {
    $form['suggested_concepts'] = array(
      '#type' => 'fieldset',
      '#title' => t('Suggested Concepts'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );

    $suggested_concepts_rows = [];
    foreach ($suggested_concepts as $suggested_concept) {
      $suggested_concepts_rows[] = [
        $suggested_concept['prefLabels'][0]['label'],
        $suggested_concept['uri']['uri'],
        format_date(strtotime($suggested_concept['date']), 'short')
      ];
    }

    $form['suggested_concepts']['concept_table'] = array(
      '#theme' => 'table',
      '#header' => array(
        t('Name'),
        t('URI'),
        t('Suggestion date'),
      ),
      '#rows' => $suggested_concepts_rows,
      '#attributes' => array(
        'id' => 'pp-taxonomy-manager-suggested-concepts-table',
        'class' => array('semantic-connector-tablesorter'),
      ),
    );

    // Add the tablesorter library if available.
    $tablesorter_path = 'sites/all/libraries/tablesorter';
    if (module_exists('libraries')) {
      $tablesorter_path = libraries_get_path('tablesorter');
    }
    if (file_exists($tablesorter_path . '/jquery.tablesorter.min.js')) {
      $form['suggested_concepts']['concept_table']['#attached']['js'] = array(libraries_get_path('tablesorter') . '/jquery.tablesorter.min.js');
    }
  }

  return $form;
}

/**
 * Ajax callback function for updating the freeterm list during the concept suggestion.
 */
function pp_taxonomy_manager_suggest_concept_list_update($form, $form_state) {
  return $form['freeterm_list'];
}

/**
 * The submit callback for the concept suggestion form.
 */
function pp_taxonomy_manager_suggest_concepts_form_validate($form, &$form_state) {
  $selected_freeterms = array_filter($form_state['values']['freeterm_list']);
  if (count($selected_freeterms) == 0) {
    form_set_error('freeterm_list', t('Select at least one free-term from the list.'));
  }
}

/**
 * The submit callback for the concept suggestion form.
 */
function pp_taxonomy_manager_suggest_concepts_form_submit($form, &$form_state) {
  $powertagging_config = $form_state['values']['powertagging_config'];
  $term_ids = array_values(array_filter($form_state['values']['freeterm_list']));
  $terms = taxonomy_term_load_multiple($term_ids);
  $suggest_concepts = [];
  foreach ($terms as $term) {
    $suggest_concepts[] = array(
      'prefLabels' => [array(
        'label' => $term->name,
        'language' => isset($term->language) ? $term->language : 'en',
      )],
    );
  }

  /** @var \SemanticConnectorPPTApi $ppt_api */
  $ppt_api = $powertagging_config->connection->getApi('PPT');
  $result = $ppt_api->suggestConcepts($powertagging_config->project_id, $suggest_concepts);
  if (empty($result)) {
    drupal_set_message(t('An error occurred while trying to suggest the concepts.'), 'error');
  }
  else {
    drupal_set_message(t('Successfully suggested the selected concepts.'));
  }
}

/**
 * Checks if the background batch is running.
 *
 * @param string|array $urls
 *   The URL of the source.
 *
 * @return int|boolean
 *   The batch ID or FALSE if the batch is not running.
 */
function pp_taxonomy_manager_batch_running($urls) {
  // Check if the Background Batch module is enabled.
  if (!module_exists('background_batch')) {
    return FALSE;
  }

  // Check if the appropriate batch is running.
  $bids = db_select('batch', 'b')
    ->fields('b', array('bid', 'batch'))
    ->orderBy('b.bid', 'ASC')
    ->execute()
    ->fetchAllKeyed(0, 1);

  if (!is_array($urls)) {
    $urls = array($urls);
  }

  foreach ($bids as $bid => $data) {
    $batch = unserialize($data);
    if (in_array($batch['source_url'], $urls)) {
      return $bid;
      break;
    }
  }

  return FALSE;
}

/**
 * Returns all selected languages with the default language first.
 *
 * @param array $all_languages
 *   An array of languages:
 *    key = Drupal language
 *    value = PoolParty language.
 *
 * @return array
 *   All mapped languages with the default language first.
 */
function pp_taxonomy_manager_order_languages($all_languages) {
  $default_language = language_default('language');
  $languages[$default_language] = $all_languages[$default_language];
  unset($all_languages[$default_language]);
  if (!empty($all_languages)) {
    foreach ($all_languages as $drupal_lang => $pp_lang) {
      if (!empty($pp_lang)) {
        $languages[$drupal_lang] = $pp_lang;
      }
    }
  }

  return $languages;
}
